@@ -1,9 +1,11 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2012-2016 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from typing import Any
+
 from xpra.net.file_transfer import FileTransferHandler
 from xpra.log import Logger
 
@@ -13,9 +15,7 @@ log = Logger()
 class FakeClient(FileTransferHandler):
     def __init__(self):
         FileTransferHandler.__init__(self)
-        self.supports_mmap = False
         self.mmap = None
-        self.mmap_enabled = False
         self.session_name = ""
         self._focused = None
         self.readonly = False
@@ -142,8 +142,8 @@ class FakeClient(FileTransferHandler):
     def request_frame_extents(self, window):
         pass
 
-    def get_window_frame_sizes(self):
-        return None
+    def get_window_frame_sizes(self) -> dict[str, Any]:
+        return {}
 
     def fsx(self, v):
         return v
