@@ -3,17 +3,19 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from collections.abc import Sequence
+
 from xpra.os_util import gi_import
-from xpra.notifications.notifier_base import NotifierBase, NID
+from xpra.notification.base import NotifierBase, NID
 
 Notify = gi_import("Notify")
 
 
 class GINotifier(NotifierBase):
 
-    def show_notify(self, dbus_id, tray, nid: NID,
-                    app_name: str, replaces_nid: NID, app_icon,
-                    summary: str, body: str, actions, hints, timeout: int, icon) -> None:
+    def show_notify(self, dbus_id: str, tray, nid: NID,
+                    app_name: str, replaces_nid: NID, app_icon: str,
+                    summary: str, body: str, actions: Sequence[str], hints: dict, timeout: int, icon) -> None:
         if not self.dbus_check(dbus_id):
             return
         icon_string = self.get_icon_string(nid, app_icon, icon)
