@@ -9,13 +9,14 @@ from xpra.os_util import gi_import
 from xpra.util.str_fn import csv
 from xpra.util.env import envbool
 from xpra.common import noop
-from xpra.gtk.widget import scaled_image
+from xpra.gtk.widget import scaled_image, checkitem
 from xpra.gtk.dialogs.about import about
 from xpra.client.gtk3.tray_menu import (
     GTKTrayMenu,
     CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_NAME_TO_LABEL, CLIPBOARD_LABELS,
     CLIPBOARD_DIRECTION_LABELS, CLIPBOARD_DIRECTION_NAME_TO_LABEL,
-    SHOW_UPLOAD, SHOW_VERSION_CHECK, RUNCOMMAND_MENU, SHOW_SERVER_COMMANDS, SHOW_SHUTDOWN,
+    SHOW_UPLOAD, SHOW_SERVER_DEBUG,
+    SHOW_VERSION_CHECK, RUNCOMMAND_MENU, SHOW_SERVER_COMMANDS, SHOW_SHUTDOWN,
     SHOW_QR,
 )
 from xpra.platform.paths import get_icon
@@ -196,7 +197,7 @@ class OSXMenuHelper(GTKTrayMenu):
                 menu.add(item)
 
         if SHOW_INFO_MENU:
-            info_menu = self.make_menu()
+            info_menu = Gtk.Menu()
             menus.append(("Info", info_menu))
             add(info_menu, self.make_sessioninfomenuitem())
             if SHOW_QR:
@@ -205,16 +206,16 @@ class OSXMenuHelper(GTKTrayMenu):
                 add(info_menu, self.make_updatecheckmenuitem())
             add(info_menu, self.make_bugreportmenuitem())
         if SHOW_FEATURES_MENU:
-            features_menu = self.make_menu()
+            features_menu = Gtk.Menu()
             menus.append(("Features", features_menu))
             self.append_featuresmenuitems(features_menu)
-            if features.windows:
+            if features.window:
                 add(features_menu, self.make_swapkeysmenuitem())
                 add(features_menu, self.make_invertmousewheelmenuitem())
                 add(features_menu, self.make_numlockmenuitem())
                 add(features_menu, self.make_scalingmenuitem())
         if features.clipboard and SHOW_CLIPBOARD_MENU:
-            clipboard_menu = self.make_menu()
+            clipboard_menu = Gtk.Menu()
             menus.append(("Clipboard", clipboard_menu))
             for label in CLIPBOARD_LABELS:
                 add(clipboard_menu, self.make_clipboard_submenuitem(label, self._remote_clipboard_changed))
@@ -224,14 +225,14 @@ class OSXMenuHelper(GTKTrayMenu):
             clipboard_menu.show_all()
             self.after_handshake(self.set_clipboard_menu, clipboard_menu)
         if features.audio and SHOW_SOUND_MENU:
-            audio_menu = self.make_menu()
+            audio_menu = Gtk.Menu()
             if self.client.speaker_allowed and self.client.speaker_codecs:
                 add(audio_menu, self.make_speakermenuitem())
             if self.client.microphone_allowed and self.client.microphone_codecs:
                 add(audio_menu, self.make_microphonemenuitem())
             menus.append(("Audio", audio_menu))
-        if features.windows and SHOW_ENCODINGS_MENU:
-            encodings_menu = self.make_menu()
+        if features.window and SHOW_ENCODINGS_MENU:
+            encodings_menu = Gtk.Menu()
 
             def set_encodings_menu(*_args):
                 client_encodings, server_encodings = self.get_encoding_options()
@@ -241,8 +242,8 @@ class OSXMenuHelper(GTKTrayMenu):
 
             self.after_handshake(set_encodings_menu)
             menus.append(("Encoding", encodings_menu))
-        if features.windows and SHOW_ACTIONS_MENU:
-            actions_menu = self.make_menu()
+        if features.window and SHOW_ACTIONS_MENU:
+            actions_menu = Gtk.Menu()
             add(actions_menu, self.make_raisewindowsmenuitem())
             add(actions_menu, self.make_minimizewindowsmenuitem())
             add(actions_menu, self.make_refreshmenuitem())
@@ -250,7 +251,7 @@ class OSXMenuHelper(GTKTrayMenu):
             self.window_menu = actions_menu
             menus.append(("Windows", actions_menu))
         if RUNCOMMAND_MENU or SHOW_SERVER_COMMANDS or SHOW_UPLOAD or SHOW_SHUTDOWN:
-            server_menu = self.make_menu()
+            server_menu = Gtk.Menu()
             if SHOW_SHUTDOWN:
                 add(server_menu, self.make_shutdownmenuitem())
 
@@ -263,6 +264,8 @@ class OSXMenuHelper(GTKTrayMenu):
                     add(server_menu, self.make_servercommandsmenuitem())
                 if SHOW_UPLOAD and self.client.remote_file_transfer:
                     add(server_menu, self.make_uploadmenuitem())
+                if SHOW_SERVER_DEBUG:
+                    add(server_menu, self.make_serverdebugmenuitem())
 
             self.after_handshake(add_ah)
             menus.append(("Server", server_menu))
@@ -295,7 +298,7 @@ class OSXMenuHelper(GTKTrayMenu):
         self.set_new_remote_clipboard(remote_clipboard)
 
     def make_clipboard_submenuitem(self, label: str, cb: Callable = noop):
-        clipboard_item = self.checkitem(label)
+        clipboard_item = checkitem(label)
         clipboard_item.set_draw_as_radio(True)
 
         def clipboard_option_changed(item):
@@ -377,7 +380,7 @@ class OSXMenuHelper(GTKTrayMenu):
             if keyboard:
                 keyboard.swap_keys = v
 
-        swapkeys_menuitem = self.checkitem("Control/Command Key Swap", swapkeys_toggled)
+        swapkeys_menuitem = checkitem("Control/Command Key Swap", swapkeys_toggled)
 
         def set_swapkeys_menuitem(*args):
             keyboard = self._get_keyboard()
@@ -402,7 +405,7 @@ class OSXMenuHelper(GTKTrayMenu):
                 self.client.wheel_map[4] = 4
                 self.client.wheel_map[5] = 5
 
-        mousewheel_menuitem = self.checkitem("Invert Mouse Wheel", invert_toggled)
+        mousewheel_menuitem = checkitem("Invert Mouse Wheel", invert_toggled)
         mousewheel_menuitem.set_active(self.client.wheel_map.get(4) != 4)
         return mousewheel_menuitem
 
@@ -414,7 +417,7 @@ class OSXMenuHelper(GTKTrayMenu):
             if keyboard:
                 keyboard.num_lock_state = v
 
-        self.numlock_menuitem = self.checkitem("Num Lock", cb=numlock_toggled)
+        self.numlock_menuitem = checkitem("Num Lock", cb=numlock_toggled)
         self.numlock_menuitem.set_active(True)
 
         def set_numlock_menuitem(*args):
@@ -435,7 +438,7 @@ class OSXMenuHelper(GTKTrayMenu):
 
     def build_dock_menu(self):
         log("OSXMenuHelper.build_dock_menu()")
-        self.dock_menu = self.make_menu()
+        self.dock_menu = Gtk.Menu()
         if SHOW_ABOUT_XPRA:
             self.dock_menu.add(self.menuitem("About Xpra", "information.png", None, about))
         self.dock_menu.show_all()
