@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -186,7 +186,7 @@ class win32NotifyIcon:
         self.app_id = app_id
         self.title = title
         self.current_icon = None
-        self.destroy_icon = None
+        self.destroy_icon = noop
         self.move_callback = move_callback
         self.click_callback = click_callback
         self.exit_callback = exit_callback
@@ -280,7 +280,7 @@ class win32NotifyIcon:
             di = self.destroy_icon
             if ci and di:
                 self.current_icon = None
-                self.destroy_icon = None
+                self.destroy_icon = noop
                 di(ci)
         except Exception as e:
             log("delete_tray_window()", exc_info=True)
