@@ -13,18 +13,18 @@ from collections.abc import Callable, Sequence
 from xpra.os_util import gi_import
 from xpra.util.env import envbool, envint, first_time
 from xpra.util.io import get_proc_cmdline
-from xpra.x11.common import Unmanageable
-from xpra.gtk.gobject import one_arg_signal, n_arg_signal
-from xpra.gtk.error import XError, xsync, xswallow, xlog
+from xpra.util.gobject import one_arg_signal, n_arg_signal
 from xpra.codecs.image import ImageWrapper
 from xpra.platform.posix.proc import get_parent_pid
-from xpra.x11.bindings.window import X11WindowBindings, constants, SHAPE_KIND
-from xpra.x11.bindings.res import ResBindings
+from xpra.server.window.model import WindowModelStub
+from xpra.x11.common import Unmanageable, X11Event, FRAME_EXTENTS
+from xpra.x11.error import XError, xsync, xswallow, xlog
+from xpra.x11.bindings.core import constants, get_root_xid
+from xpra.x11.bindings.window import X11WindowBindings
 from xpra.x11.bindings.send_wm import send_wm_delete_window
-from xpra.x11.models.model_stub import WindowModelStub
-from xpra.x11.gtk.composite import CompositeHelper
-from xpra.x11.gtk.prop import prop_get, prop_set, prop_del, prop_type_get, PYTHON_TYPES
-from xpra.x11.gtk.bindings import add_event_receiver, remove_event_receiver, get_pywindow
+from xpra.x11.xroot_props import root_get, root_set
+from xpra.x11.prop import prop_get, prop_set, prop_del, prop_type_get, PYTHON_TYPES
+from xpra.x11.dispatch import add_event_receiver, remove_event_receiver
 from xpra.log import Logger
 
 log = Logger("x11", "window")
@@ -33,23 +33,28 @@ shapelog = Logger("x11", "window", "shape")
 grablog = Logger("x11", "window", "grab")
 framelog = Logger("x11", "window", "frame")
 geomlog = Logger("x11", "window", "geometry")
+belllog = Logger("x11", "window", "bell")
 
 GObject = gi_import("GObject")
 GLib = gi_import("GLib")
 
 X11Window = X11WindowBindings()
 
-XRes = ResBindings()
-if not XRes.check_xres():
-    log.warn("Warning: X Resource Extension missing or too old")
+try:
+    from xpra.x11.bindings.res import ResBindings
+    XRes = ResBindings()
+except ImportError:
+    log.warn("Warning: missing X Resource Extension bindings")
     XRes = None
-
-if get_parent_pid is None:
-    log("proc.get_parent_pid is not available")
+else:
+    if not XRes.check_xres():
+        log.warn("Warning: X Resource Extension missing or too old")
+        XRes = None
+if XRes is None:
+    log.warn(" the process IDs cannot be trusted")
 
 FORCE_QUIT = envbool("XPRA_FORCE_QUIT", True)
 XSHAPE = envbool("XPRA_XSHAPE", True)
-FRAME_EXTENTS = envbool("XPRA_FRAME_EXTENTS", True)
 OPAQUE_REGION = envbool("XPRA_OPAQUE_REGION", True)
 DELETE_DESTROY = envbool("XPRA_DELETE_DESTROY", False)
 DELETE_KILL_PID = envbool("XPRA_DELETE_KILL_PID", XRes is not None)
@@ -108,6 +113,8 @@ X11PROPERTY_SYNC_BLOCKLIST = os.environ.get("XPRA_X11PROPERTY_SYNC_BLOCKLIST",
                                             "_GTK,WM_,_NET,Xdnd").split(",")
 SHAPE_DELAY = envint("XPRA_SHAPE_DELAY", 100)
 
+DEFAULT_ACTIONS = tuple(x for x in os.environ.get("XPRA_WINDOW_DEFAULT_ACTIONS", "").split(",") if x.strip())
+
 
 def sanestr(s: str) -> str:
     return s.strip("\0").replace("\0", " ")
@@ -131,8 +138,9 @@ class CoreX11WindowModel(WindowModelStub):
             GObject.ParamFlags.READABLE,
         ),
         "parent": (
-            GObject.TYPE_PYOBJECT,
-            "parent window id", "",
+            GObject.TYPE_INT,
+            "parent X11 window id", "",
+            -1, 65535, -1,
             GObject.ParamFlags.READABLE,
         ),
         # FIXME: this is an ugly virtual property
@@ -148,7 +156,7 @@ class CoreX11WindowModel(WindowModelStub):
             -1, 64, -1,
             GObject.ParamFlags.READABLE,
         ),
-        # if the window depth is 32 bit
+        # is there an alpha channel?
         "has-alpha": (
             GObject.TYPE_BOOLEAN,
             "Does the window use transparency", "",
@@ -184,7 +192,13 @@ class CoreX11WindowModel(WindowModelStub):
         # from _NET_WM_NAME or WM_NAME
         "title": (
             GObject.TYPE_PYOBJECT,
-            "Window title (unicode or None)", "",
+            "Window title", "",
+            GObject.ParamFlags.READABLE,
+        ),
+        # from WM_LOCALE_NAME
+        "locale": (
+            GObject.TYPE_PYOBJECT,
+            "The name of the current locale used by the application", "",
             GObject.ParamFlags.READABLE,
         ),
         # from WM_WINDOW_ROLE
@@ -235,6 +249,11 @@ class CoreX11WindowModel(WindowModelStub):
             "Compositor can assume that there is no transparency for this region", "",
             GObject.ParamFlags.READWRITE,
         ),
+        "actions": (
+            GObject.TYPE_PYOBJECT,
+            "Custom window actions", "",
+            GObject.ParamFlags.READWRITE,
+        ),
     }
 
     __common_signals__ = {
@@ -247,6 +266,7 @@ class CoreX11WindowModel(WindowModelStub):
         "bell": one_arg_signal,
         "client-contents-changed": one_arg_signal,
         "motion": one_arg_signal,
+        "action": one_arg_signal,
         # x11 events we catch (and often re-emit as something else):
         "x11-property-notify-event": one_arg_signal,
         "x11-xkb-event": one_arg_signal,
@@ -265,14 +285,14 @@ class CoreX11WindowModel(WindowModelStub):
         "xid", "depth", "has-alpha",
         "parent",
         "client-machine", "pid", "ppid", "wm-pid",
-        "title", "role",
+        "title", "locale", "role",
         "command", "shape",
         "class-instance", "protocols",
-        "opaque-region",
+        "opaque-region", "actions",
     ]
     # exposed and changing (should be watched for notify signals):
     _dynamic_property_names = [
-        "title", "command", "shape", "class-instance", "protocols", "opaque-region",
+        "title", "command", "shape", "class-instance", "protocols", "opaque-region", "actions",
     ]
     # should not be exported to the clients:
     _internal_property_names = [
@@ -282,6 +302,7 @@ class CoreX11WindowModel(WindowModelStub):
         "_NET_WM_PID", "WM_CLIENT_MACHINE",
         # _NET_WM_NAME is redundant, as it calls the same handler as "WM_NAME"
         "WM_NAME", "_NET_WM_NAME",
+        "WM_LOCALE_NAME",
         "WM_PROTOCOLS", "WM_CLASS", "WM_WINDOW_ROLE",
         "_NET_WM_OPAQUE_REGION",
         "WM_COMMAND",
@@ -300,10 +321,10 @@ class CoreX11WindowModel(WindowModelStub):
             raise TypeError(f"xid must be an int, not a {type(xid)}")
         log("new window %#x", xid)
         self.xid: int = xid
-        self._composite: CompositeHelper | None = None
+        self._composite = None
         self._damage_forward_handle = None
-        self._setup_done = False
         self._kill_count = 0
+        self._xshape = False
         self._shape_timer = 0
         self._shape_timer_serial = 0
         self._updateprop("xid", self.xid)
@@ -341,24 +362,42 @@ class CoreX11WindowModel(WindowModelStub):
             log("failed to manage %#x", self.xid, exc_info=True)
             raise Unmanageable(e) from e
         add_event_receiver(self.xid, self)
-        # Keith Packard says that composite state is undefined following a
-        # reparent, so I'm not sure doing this here in the superclass,
-        # before we reparent, actually works... let's wait and see.
         try:
-            self._composite = CompositeHelper(self.xid)
-            with xsync:
-                self._composite.setup()
-                if X11Window.displayHasXShape():
-                    X11Window.XShapeSelectInput(self.xid)
-        except Exception as e:
-            remove_event_receiver(self.xid, self)
-            log("%s %#x does not support compositing: %s", self._MODELTYPE, self.xid, e)
-            with xswallow:
-                self._composite.destroy()
-            self._composite = None
-            if isinstance(e, Unmanageable):
-                raise
-            raise Unmanageable(e) from e
+            from xpra.x11.composite import CompositeHelper
+        except ImportError:
+            from xpra.util.env import first_time
+            if first_time("composite-helper"):
+                log.error("Error: unable to load the composite bindings")
+                log.error(" the window contents will not be visible")
+        else:
+            # Keith Packard says that composite state is undefined following a
+            # reparent, so I'm not sure doing this here in the superclass,
+            # before we reparent, actually works... let's wait and see.
+            try:
+                self._composite = CompositeHelper(self.xid)
+                with xsync:
+                    self._composite.setup()
+            except Exception as e:
+                remove_event_receiver(self.xid, self)
+                log("%s %#x does not support compositing: %s", self._MODELTYPE, self.xid, e)
+                with xswallow:
+                    self._composite.destroy()
+                self._composite = None
+                if isinstance(e, Unmanageable):
+                    raise
+                raise Unmanageable(e) from e
+        with xlog:
+            try:
+                from xpra.x11.bindings.shape import XShapeBindings
+                XShape = XShapeBindings()
+            except ImportError as e:
+                log.warn("Warning: unable to load the X11 Shape extension")
+                log.warn(" %s", e)
+                log.warn(" window shapes cannot be forwarded")
+            else:
+                if XShape.hasXShape():
+                    self._xshape = True
+                    XShape.XShapeSelectInput(self.xid)
         # compositing is now enabled,
         # from now on we must call setup_failed to clean things up
         self._managed = True
@@ -382,14 +421,16 @@ class CoreX11WindowModel(WindowModelStub):
     def setup(self) -> None:
         # Start listening for important events.
         X11Window.addDefaultEvents(self.xid)
-        self._damage_forward_handle = self._composite.connect("contents-changed", self._forward_contents_changed)
+        c = self._composite
+        if c:
+            self._damage_forward_handle = c.connect("contents-changed", self._forward_contents_changed)
         self._setup_property_sync()
 
     def unmanage(self, exiting=False) -> None:
         if self._managed:
             self.emit("unmanaged", exiting)
 
-    def do_unmanaged(self, wm_exiting) -> None:
+    def do_unmanaged(self, wm_exiting: bool) -> None:
         if not self._managed:
             return
         self._managed = False
@@ -414,11 +455,12 @@ class CoreX11WindowModel(WindowModelStub):
         if not self._managed:
             return
         c = self._composite
-        if not c:
-            raise RuntimeError("composite window destroyed outside the UI thread?")
-        c.acknowledge_changes()
+        if c:
+            c.acknowledge_changes()
+        else:
+            log("composite window destroyed outside the UI thread?")
 
-    def _forward_contents_changed(self, _obj, event) -> None:
+    def _forward_contents_changed(self, _obj, event: X11Event) -> None:
         if self._managed:
             self.emit("client-contents-changed", event)
 
@@ -426,8 +468,11 @@ class CoreX11WindowModel(WindowModelStub):
         c = self._composite
         return bool(c) and c.has_xshm()
 
-    def get_image(self, x: int, y: int, width: int, height: int) -> ImageWrapper:
-        return self._composite.get_image(x, y, width, height)
+    def get_image(self, x: int, y: int, width: int, height: int) -> ImageWrapper | None:
+        c = self._composite
+        if not c:
+            return None
+        return c.get_image(x, y, width, height)
 
     def _setup_property_sync(self) -> None:
         metalog("setup_property_sync()")
@@ -454,10 +499,10 @@ class CoreX11WindowModel(WindowModelStub):
         metalog("read_initial_X11_properties() core")
         # immutable ones:
         depth = X11Window.get_depth(self.xid)
-        pid = XRes.get_pid(self.xid) if XRes else -1
-        ppid = get_parent_pid(pid) if pid > 0 and get_parent_pid else 0
+        pid = XRes.get_pid(self.xid) if XRes else 0
+        ppid = get_parent_pid(pid) if pid else 0
         parent = X11Window.getParent(self.xid)
-        if parent == X11Window.get_root_xid():
+        if parent == get_root_xid():
             parent = 0
         metalog("initial X11 properties: xid=%#x, parent=%#x, depth=%i, pid=%i, ppid=%i",
                 self.xid, parent, depth, pid, ppid)
@@ -467,7 +512,8 @@ class CoreX11WindowModel(WindowModelStub):
         self._updateprop("has-alpha", depth == 32)
         self._updateprop("allowed-actions", self._DEFAULT_NET_WM_ALLOWED_ACTIONS)
         self._updateprop("shape", self._read_xshape())
-        self._updateprop("parent", get_pywindow(parent))
+        self._updateprop("parent", parent)
+        self._updateprop("actions", DEFAULT_ACTIONS)
         # note: some of those are technically mutable,
         # but we don't export them as "dynamic" properties, so this won't be propagated
         # maybe we want to catch errors parsing _NET_WM_ICON ?
@@ -489,7 +535,7 @@ class CoreX11WindowModel(WindowModelStub):
                     raise
                 except Exception:
                     # try to continue:
-                    log.error("Error parsing initial property {mutable!r}", exc_info=True)
+                    log.error(f"Error parsing initial property {mutable!r}", exc_info=True)
 
     def _scrub_x11(self) -> None:
         metalog("scrub_x11() x11 properties=%s", self._scrub_x11_properties)
@@ -503,8 +549,10 @@ class CoreX11WindowModel(WindowModelStub):
     # XShape
     #########################################
 
-    def read_xshape(self):
+    def read_xshape(self) -> None:
         shapelog("read_xshape()")
+        if not self._xshape:
+            return
         self._shape_timer = 0
         cur_shape = self.get_property("shape") or {}
         # remove serial before comparing dicts:
@@ -522,9 +570,14 @@ class CoreX11WindowModel(WindowModelStub):
             self._internal_set_property("shape", v)
 
     def _read_xshape(self, x: int = 0, y: int = 0) -> dict[str, Any]:
-        if not X11Window.displayHasXShape() or not XSHAPE:
+        if not self._xshape:
             return {}
-        extents = X11Window.XShapeQueryExtents(self.xid)
+        from xpra.x11.bindings.shape import init_xshape_events, XShapeBindings, SHAPE_KIND
+        XShape = XShapeBindings()
+        if not XShape.hasXShape():
+            return {}
+        init_xshape_events()
+        extents = XShape.XShapeQueryExtents(self.xid)
         if not extents:
             shapelog("read_shape for window %#x: no extents", self.xid)
             return {}
@@ -551,11 +604,11 @@ class CoreX11WindowModel(WindowModelStub):
     # Property reading
     ################################
 
-    def get_dimensions(self):
+    def get_dimensions(self) -> tuple[int, int]:
         # just extracts the size from the geometry:
         return self.get_property("geometry")[2:4]
 
-    def get_geometry(self):
+    def get_geometry(self) -> tuple[int, int, int, int]:
         return self.get_property("geometry")[:4]
 
     #########################################
@@ -578,7 +631,7 @@ class CoreX11WindowModel(WindowModelStub):
         v = self.get_property("frame")
         framelog("sync_frame: frame(%#x)=%s", self.xid, v)
         if not v and (not self.is_OR() and not self.is_tray()):
-            v = self.root_prop_get("DEFAULT_NET_FRAME_EXTENTS", ["u32"])
+            v = root_get("DEFAULT_NET_FRAME_EXTENTS", ["u32"])
         if not v:
             # default for OR, or if we don't have any other value:
             v = (0, 0, 0, 0)
@@ -611,13 +664,7 @@ class CoreX11WindowModel(WindowModelStub):
     def prop_set(self, key: str, ptype, value) -> None:
         prop_set(self.xid, key, ptype, value)
 
-    def root_prop_get(self, key: str, ptype, ignore_errors=True) -> object:
-        return prop_get(X11Window.get_root_xid(), key, ptype, ignore_errors=ignore_errors)
-
-    def root_prop_set(self, key: str, ptype, value) -> None:
-        prop_set(X11Window.get_root_xid(), key, ptype, value)
-
-    def do_x11_property_notify_event(self, event) -> None:
+    def do_x11_property_notify_event(self, event: X11Event) -> None:
         # X11: PropertyNotify
         assert event.window == self.xid
         self._handle_property_change(str(event.atom))
@@ -680,7 +727,13 @@ class CoreX11WindowModel(WindowModelStub):
             name = self.prop_get("WM_NAME", "latin1", True) or ""
             metalog("WM_NAME=%s", name)
         if self._updateprop("title", sanestr(str(name) or "")):
-            metalog("wm_name changed")
+            metalog("wm_name changed: %r", name)
+
+    def _handle_wm_local_name_change(self) -> None:
+        locale = self.prop_get("WM_LOCALE_NAME", "latin1", True)
+        metalog("WM_LOCALE_NAME=%s", locale)
+        if self._updateprop("locale", sanestr(str(locale) or "")):
+            metalog("locale changed: %r", locale)
 
     def _handle_role_change(self) -> None:
         role = self.prop_get("WM_WINDOW_ROLE", "latin1") or ""
@@ -704,11 +757,18 @@ class CoreX11WindowModel(WindowModelStub):
         self._updateprop("command", command)
 
     def _handle_class_change(self) -> None:
-        class_instance = X11Window.getClassHint(self.xid)
-        if class_instance:
-            class_instance = tuple(v.decode("latin1") for v in class_instance)
+        try:
+            from xpra.x11.bindings.classhint import XClassHintBindings
+            class_instance = XClassHintBindings().getClassHint(self.xid)
+            if class_instance:
+                class_instance = tuple(v.decode("latin1") for v in class_instance)
+        except ImportError as e:
+            if first_time("x11-class-hint"):
+                log.warn("Warning: window class-hint is not available")
+                log.warn(" %s", e)
+            class_instance = ()
         metalog("WM_CLASS=%s", class_instance)
-        self._updateprop("class-instance", class_instance or ())
+        self._updateprop("class-instance", class_instance or ("", ""))
 
     def _handle_opaque_region_change(self) -> None:
         rectangles = []
@@ -721,28 +781,38 @@ class CoreX11WindowModel(WindowModelStub):
         metalog("_NET_WM_OPAQUE_REGION(%s)=%s (OPAQUE_REGION=%s)", v, rectangles, OPAQUE_REGION)
         self._updateprop("opaque-region", tuple(rectangles))
 
+    def _handle_user_time_window(self) -> None:
+        xid = self.prop_get("_NET_WM_USER_TIME_WINDOW", "window") or -1
+        log("_NET_WM_USER_TIME_WINDOW=%#x", xid)
+        time = -1
+        if xid > 0:
+            time = prop_get(xid, "_NET_WM_USER_TIME", "u32") or -1
+        log("_NET_WM_USER_TIME(%#x)=%i", xid, time)
+
     # these handlers must not generate X11 errors (must use XSync)
     _x11_property_handlers: dict[str, Callable] = {
         "_NET_WM_PID": _handle_pid_change,
         "WM_CLIENT_MACHINE": _handle_client_machine_change,
         "WM_NAME": _handle_wm_name_change,
+        "WM_LOCALE_NAME": _handle_wm_local_name_change,
         "_NET_WM_NAME": _handle_wm_name_change,
         "WM_WINDOW_ROLE": _handle_role_change,
         "WM_PROTOCOLS": _handle_protocols_change,
         "WM_COMMAND": _handle_command_change,
         "WM_CLASS": _handle_class_change,
         "_NET_WM_OPAQUE_REGION": _handle_opaque_region_change,
+        "_NET_WM_USER_TIME_WINDOW": _handle_user_time_window,
     }
 
     #########################################
     # X11 Events
     #########################################
 
-    def do_x11_unmap_event(self, event) -> None:
+    def do_x11_unmap_event(self, event: X11Event) -> None:
         log("do_x11_unmap_event(%s) xid=%s, ", event, self.xid)
         self.unmanage()
 
-    def do_x11_destroy_event(self, event) -> None:
+    def do_x11_destroy_event(self, event: X11Event) -> None:
         log("do_x11_destroy_event(%s) xid=%s, ", event, self.xid)
         if event.delivered_to == self.xid:
             # This is somewhat redundant with the unmap signal, because if you
@@ -753,7 +823,7 @@ class CoreX11WindowModel(WindowModelStub):
             # the morning.  It makes for simple code:
             self.unmanage()
 
-    def process_client_message_event(self, event) -> bool:
+    def process_client_message_event(self, event: X11Event) -> bool:
         # FIXME
         # Need to listen for:
         #   _NET_CURRENT_DESKTOP
@@ -784,7 +854,7 @@ class CoreX11WindowModel(WindowModelStub):
         # not handled:
         return False
 
-    def do_x11_configure_event(self, event) -> None:
+    def do_x11_configure_event(self, event: X11Event) -> None:
         if not self._managed:
             return
         # shouldn't the border width always be 0?
@@ -793,7 +863,8 @@ class CoreX11WindowModel(WindowModelStub):
                 event, self.xid, geom)
         self._updateprop("geometry", geom)
 
-    def do_x11_shape_event(self, event) -> None:
+    def do_x11_shape_event(self, event: X11Event) -> None:
+        from xpra.x11.bindings.shape import SHAPE_KIND
         shapelog("shape event: %s, kind=%s, timer=%s",
                  event, SHAPE_KIND.get(event.kind, event.kind), self._shape_timer)
         cur_shape = self.get_property("shape")
@@ -808,17 +879,17 @@ class CoreX11WindowModel(WindowModelStub):
             self._shape_timer = GLib.timeout_add(SHAPE_DELAY, self.read_xshape)
         self._shape_timer_serial = max(self._shape_timer_serial, serial)
 
-    def do_x11_xkb_event(self, event) -> None:
+    def do_x11_xkb_event(self, event: X11Event) -> None:
         # X11: XKBNotify
-        log("WindowModel.do_x11_xkb_event(%r)", event)
+        belllog("WindowModel.do_x11_xkb_event(%r)", event)
         if event.subtype != "bell":
-            log("WindowModel.do_x11_xkb_event(%r)", event, exc_info=True)
-            log.error("Error: unknown xkb event type: %s", event.type)
+            belllog("WindowModel.do_x11_xkb_event(%r)", event, exc_info=True)
+            belllog.error("Error: unknown xkb event type: %s", event.type)
             return
         event.window_model = self
         self.emit("bell", event)
 
-    def do_x11_client_message_event(self, event) -> None:
+    def do_x11_client_message_event(self, event: X11Event) -> None:
         # X11: ClientMessage
         log("do_x11_client_message_event(%s)", event)
         if not event.data or len(event.data) != 5:
@@ -827,7 +898,7 @@ class CoreX11WindowModel(WindowModelStub):
         if not self.process_client_message_event(event):
             log.warn("do_x11_client_message_event(%s) not handled", event)
 
-    def do_x11_focus_in_event(self, event) -> None:
+    def do_x11_focus_in_event(self, event: X11Event) -> None:
         # X11: FocusIn
         grablog("focus_in_event(%s) mode=%s, detail=%s",
                 event, GRAB_CONSTANTS.get(event.mode), DETAIL_CONSTANTS.get(event.detail, event.detail))
@@ -836,13 +907,13 @@ class CoreX11WindowModel(WindowModelStub):
         else:
             self.may_emit_grab(event)
 
-    def do_x11_focus_out_event(self, event) -> None:
+    def do_x11_focus_out_event(self, event: X11Event) -> None:
         # X11: FocusOut
         grablog("focus_out_event(%s) mode=%s, detail=%s",
                 event, GRAB_CONSTANTS.get(event.mode), DETAIL_CONSTANTS.get(event.detail, event.detail))
         self.may_emit_grab(event)
 
-    def may_emit_grab(self, event) -> None:
+    def may_emit_grab(self, event: X11Event) -> None:
         if event.mode == NotifyGrab:
             grablog("emitting grab on %s", self)
             self.emit("grab", event)
@@ -850,7 +921,7 @@ class CoreX11WindowModel(WindowModelStub):
             grablog("emitting ungrab on %s", self)
             self.emit("ungrab", event)
 
-    def do_x11_motion_event(self, event) -> None:
+    def do_x11_motion_event(self, event: X11Event) -> None:
         self.emit("motion", event)
 
     ################################
@@ -861,7 +932,7 @@ class CoreX11WindowModel(WindowModelStub):
         X11Window.XRaiseWindow(self.xid)
 
     def set_active(self) -> None:
-        self.root_prop_set("_NET_ACTIVE_WINDOW", "u32", self.xid)
+        root_set("_NET_ACTIVE_WINDOW", "u32", self.xid)
 
     ################################
     # Killing clients:
@@ -900,15 +971,13 @@ class CoreX11WindowModel(WindowModelStub):
                 return
         machine = self.get_property("client-machine")
         pid = self.get_property("pid")
-        if pid <= 0:
-            # we could fall back to _NET_WM_PID
-            # but that would be unsafe
-            log.warn("Warning: cannot terminate window %#x, no pid found", self.xid)
+        if pid <= 0 and XRes:
+            log.warn("Warning: no 'pid' for window %#x", self.xid)
             if machine:
-                log.warn(" WM_CLIENT_MACHINE=%s", machine)
+                log.warn(" WM_CLIENT_MACHINE=%r", machine)
             pid = self.get_property("wm-pid")
             if pid > 0:
-                log.warn(" _NET_WM_PID=%s", pid)
+                log.warn(" using _NET_WM_PID=%s", pid)
         if pid == os.getpid():
             log.warn("Warning: force_quit is refusing to kill ourselves!")
             return
