@@ -11,17 +11,18 @@ from xpra.os_util import gi_import
 from xpra.util.objects import typedict
 from xpra.util.env import envint, envbool
 from xpra.common import MAX_WINDOW_SIZE
-from xpra.gtk.gobject import one_arg_signal
-from xpra.gtk.error import XError, xsync, xswallow, xlog
-from xpra.x11.gtk.prop import prop_set
+from xpra.util.gobject import one_arg_signal
+from xpra.x11.error import XError, xsync, xswallow, xlog
+from xpra.x11.prop import prop_set
 from xpra.x11.prop_conv import MotifWMHints
+from xpra.x11.bindings.core import constants, get_root_xid
 from xpra.x11.bindings.window import X11WindowBindings
 from xpra.x11.bindings.send_wm import send_wm_take_focus
-from xpra.x11.common import Unmanageable
+from xpra.x11.common import Unmanageable, X11Event
 from xpra.x11.models.size_hints_util import sanitize_size_hints
-from xpra.x11.models.base import BaseWindowModel, constants
+from xpra.x11.models.base import BaseWindowModel
 from xpra.x11.models.core import sanestr
-from xpra.x11.gtk.bindings import add_event_receiver, remove_event_receiver
+from xpra.x11.dispatch import add_event_receiver, remove_event_receiver
 from xpra.log import Logger
 
 log = Logger("x11", "window")
@@ -198,7 +199,7 @@ class WindowModel(BaseWindowModel):
             GObject.ParamFlags.READWRITE,
         ),
         # from WM_NORMAL_HINTS
-        "size-hints": (
+        "size-constraints": (
             GObject.TYPE_PYOBJECT,
             "Client hints on constraining its size", "",
             GObject.ParamFlags.READABLE,
@@ -236,12 +237,12 @@ class WindowModel(BaseWindowModel):
     }
 
     _property_names = BaseWindowModel._property_names + [
-        "size-hints", "icon-title", "icons", "decorations",
+        "size-constraints", "icon-title", "icons", "decorations",
         "modal", "set-initial-position", "requested-position",
         "iconic",
     ]
     _dynamic_property_names = BaseWindowModel._dynamic_property_names + [
-        "size-hints", "icon-title", "icons", "decorations", "modal", "iconic",
+        "size-constraints", "icon-title", "icons", "decorations", "modal", "iconic",
     ]
     _initial_x11_properties = BaseWindowModel._initial_x11_properties + [
         "WM_HINTS", "WM_NORMAL_HINTS", "_MOTIF_WM_HINTS",
@@ -300,9 +301,15 @@ class WindowModel(BaseWindowModel):
             log("hiding inherited window")
             self.last_unmap_serial = X11Window.Unmap(self.xid)
 
-        log("setup() adding to save set")
-        X11Window.XAddToSaveSet(self.xid)
-        self.in_save_set = True
+        try:
+            log("setup() adding to save set")
+            from xpra.x11.bindings.saveset import XSaveSetBindings
+            XSaveSetBindings().XAddToSaveSet(self.xid)
+            self.in_save_set = True
+        except ImportError as e:
+            log.warn("Warning: unable to use SaveSet X11 bindings")
+            log.warn(" %s", e)
+            log.warn(" you may not be able to recover sessions after xpra exits")
 
         log("setup() reparenting")
         X11Window.Reparent(self.xid, self.corral_xid, 0, 0)
@@ -317,7 +324,7 @@ class WindowModel(BaseWindowModel):
         # use the coordinates of the corral window:
         if nx == ny == 0:
             nx, ny = x, y
-        hints = self.get_property("size-hints")
+        hints = self.get_property("size-constraints")
         geomlog("setup() hints=%s size=%ix%i", hints, w, h)
         nw, nh = self.calc_constrained_size(w, h, hints)
         self._updateprop("geometry", (nx, ny, nw, nh))
@@ -422,7 +429,7 @@ class WindowModel(BaseWindowModel):
         self._internal_set_property("requested-size", (aw, ah))
         self.update_children()
 
-    def do_unmanaged(self, wm_exiting) -> None:
+    def do_unmanaged(self, wm_exiting: bool) -> None:
         log("unmanaging window: %s (%s - %s)", self, self.corral_xid, self.xid)
         cxid = self.corral_xid
         if cxid:
@@ -436,7 +443,7 @@ class WindowModel(BaseWindowModel):
             if geom is not None:
                 with xswallow:
                     if self.client_reparented:
-                        X11Window.Reparent(self.xid, X11Window.get_root_xid(), 0, 0)
+                        X11Window.Reparent(self.xid, get_root_xid(), 0, 0)
                     if self.saved_events != -1:
                         with xswallow:
                             X11Window.setEventMask(self.xid, self.saved_events)
@@ -448,8 +455,9 @@ class WindowModel(BaseWindowModel):
             # section 10. Connection Close).  This causes "ghost windows", see
             # bug #27:
             if self.in_save_set:
+                from xpra.x11.bindings.saveset import XSaveSetBindings
                 with xswallow:
-                    X11Window.XRemoveFromSaveSet(self.xid)
+                    XSaveSetBindings().XRemoveFromSaveSet(self.xid)
                 self.in_save_set = False
             if geom:
                 self.send_configure_notify()
@@ -491,13 +499,13 @@ class WindowModel(BaseWindowModel):
     # X11 Events
     #########################################
 
-    def do_x11_property_notify_event(self, event) -> None:
+    def do_x11_property_notify_event(self, event: X11Event) -> None:
         if event.delivered_to == self.corral_xid:
             return
         super().do_x11_property_notify_event(event)
 
     # noinspection PyMethodMayBeStatic
-    def do_x11_child_map_request_event(self, event) -> None:
+    def do_x11_child_map_request_event(self, event: X11Event) -> None:
         # If we get a MapRequest then it might mean that someone tried to map
         # this window multiple times in quick succession, before we actually
         # mapped it (so that several MapRequests ended up queued up; FSF Emacs
@@ -507,7 +515,7 @@ class WindowModel(BaseWindowModel):
         # request.
         log("do_x11_child_map_request_event(%s)", event)
 
-    def do_x11_unmap_event(self, event) -> None:
+    def do_x11_unmap_event(self, event: X11Event) -> None:
         log(f"do_x11_unmap_event({event}) corral_xid={self.corral_xid:x}")
         if not self.corral_xid or event.delivered_to == self.corral_xid:
             return
@@ -525,7 +533,7 @@ class WindowModel(BaseWindowModel):
         if event.send_event or self.serial_after_last_unmap(event.serial):
             self.unmanage()
 
-    def do_x11_destroy_event(self, event) -> None:
+    def do_x11_destroy_event(self, event: X11Event) -> None:
         log(f"do_x11_destroy_event({event}) corral_xid={self.corral_xid:x}")
         if not self.corral_xid or event.delivered_to == self.corral_xid:
             return
@@ -584,7 +592,7 @@ class WindowModel(BaseWindowModel):
 
     def _do_update_client_geometry(self, x, y, allocated_w, allocated_h) -> None:
         geomlog("_do_update_client_geometry(%s)", (x, y, allocated_w, allocated_h))
-        hints = self.get_property("size-hints")
+        hints = self.get_property("size-constraints")
         w, h = self.calc_constrained_size(allocated_w, allocated_h, hints)
         geomlog("_do_update_client_geometry: size(%s)=%ix%i", hints, w, h)
         with xlog:
@@ -604,7 +612,7 @@ class WindowModel(BaseWindowModel):
                 X11Window.sendConfigureNotify(self.xid)
             self._updateprop("geometry", (x, y, w, h))
 
-    def do_x11_configure_event(self, event) -> None:
+    def do_x11_configure_event(self, event: X11Event) -> None:
         geomlog("WindowModel.do_x11_configure_event(%s) corral=%#x, client=%#x, managed=%s",
                 event, self.corral_xid, self.xid, self._managed)
         if not self._managed or not self.corral_xid or not self.xid:
@@ -654,7 +662,7 @@ class WindowModel(BaseWindowModel):
                 continue
             # record xid and geometry:
             children.append([xid] + list(geom))
-        self._internal_set_property("children", children)
+        self._internal_set_property("children", tuple(children))
 
     def configure_geometry(self, x: int, y: int, w: int, h: int) -> bool:
         # the client window may have been resized or moved (generally programmatically)
@@ -662,7 +670,7 @@ class WindowModel(BaseWindowModel):
         geomlog("configure_geometry%s", (x, y, w, h))
         cow, coh = X11Window.getGeometry(self.corral_xid)[2:4]
         # size changes (and position if any):
-        hints = self.get_property("size-hints")
+        hints = self.get_property("size-constraints")
         w, h = self.calc_constrained_size(w, h, hints)
         cx, cy, cw, ch = self.get_property("geometry")
         resized = cow != w or coh != h
@@ -690,9 +698,9 @@ class WindowModel(BaseWindowModel):
             y = cy
         return self._updateprop("geometry", (x, y, w, h))
 
-    def do_x11_child_configure_request_event(self, event) -> None:
-        hints = self.get_property("size-hints")
-        geomlog("do_x11_child_configure_request_event(%s) client=%#x, corral=%#x, value_mask=%s, size-hints=%s",
+    def do_x11_child_configure_request_event(self, event: X11Event) -> None:
+        hints = self.get_property("size-constraints")
+        geomlog("do_x11_child_configure_request_event(%s) client=%#x, corral=%#x, value_mask=%s, size-constraints=%s",
                 event, self.xid, self.corral_xid, configure_bits(event.value_mask), hints)
         if event.value_mask & CWStackMode:
             geomlog(" restack above=%s, detail=%s", event.above, event.detail)
@@ -753,7 +761,7 @@ class WindowModel(BaseWindowModel):
         # (In particular, I believe that a request to jump to the top is
         # meaningful and should perhaps even be respected.)
 
-    def process_client_message_event(self, event) -> bool:
+    def process_client_message_event(self, event: X11Event) -> bool:
         if event.message_type == "_NET_MOVERESIZE_WINDOW":
             # TODO: honour gravity, show source indication
             with xsync:
@@ -771,7 +779,7 @@ class WindowModel(BaseWindowModel):
                     self._internal_set_property("set-initial-position", True)
                     self._internal_set_property("requested-position", (x, y))
                 # honour hints:
-                hints = self.get_property("size-hints")
+                hints = self.get_property("size-constraints")
                 w, h = self.calc_constrained_size(w, h, hints)
                 geomlog("_NET_MOVERESIZE_WINDOW on %s (data=%s, current geometry=%s, new geometry=%s)",
                         self, event.data, geom, (x, y, w, h))
@@ -892,7 +900,7 @@ class WindowModel(BaseWindowModel):
         # gets no-op updated -- some apps like FSF Emacs 21 like to update
         # their properties every time they see a ConfigureNotify, and this
         # reduces the chance for us to get caught in loops:
-        if self._updateprop("size-hints", hints):
+        if self._updateprop("size-constraints", hints):
             metalog("updated: size-hints=%s", hints)
             self._update_client_geometry()
 
@@ -915,7 +923,7 @@ class WindowModel(BaseWindowModel):
         return None
 
     def get_wm_state(self, prop: str) -> bool:
-        state_names = self._state_properties.get(prop)
+        state_names = self._state_properties.get(prop, ())
         assert state_names, f"invalid window state {prop}"
         log("get_wm_state(%s) state_names=%s", prop, state_names)
         # this is a virtual property for _NET_WM_STATE:
@@ -927,8 +935,7 @@ class WindowModel(BaseWindowModel):
     ################################
 
     def give_client_focus(self) -> None:
-        """The focus manager has decided that our client should receive X
-        focus.  See world_window.py for details."""
+        """The focus manager has decided that our client should receive X focus."""
         log("give_client_focus() corral_xid=%s", self.corral_xid)
         if self.corral_xid:
             with xlog:
